import docopt, os, sequtils, tempfile

import strawboss/private/util
import strawboss/configuration
import strawboss/core
import strawboss/server

let SB_VER = "0.2.0"

proc logProcOutput*(outMsg, errMsg: TaintedString, cmd: string) =
  let prefix = if cmd != nil: cmd else: ""
  if outMsg != nil: echo prefix & "(stdout): " & outMsg
  if errMsg != nil: echo prefix & "(stderr): " & errMsg

when isMainModule:

  var cfg = loadStrawBossConfig("strawboss.config.json")
  if not existsDir(cfg.artifactsRepo):
    echo "Artifacts repo (" & cfg.artifactsRepo & ") does not exist. Creating..."
    createDir(cfg.artifactsRepo)

  cfg.artifactsRepo = expandFilename(cfg.artifactsRepo)
  
  let doc = """
Usage:
  strawboss serve
  strawboss run <project> <step> [options]
  
Options
  
  -f --force-rebuild          Force a build step to re-run even we have cached
                              results from building that step before for this
                              version of the project.

  -r --reference <ref>        Build the project at this commit reference.

  -w --workspace <workspace>  Use the given directory as the build workspace.

"""

  let args = docopt(doc, version = "strawboss v" & SB_VER)

  echo $args
  if args["run"]:

    let req = RunRequest(
      projectName: $args["<project>"],
      stepName: $args["<step>"],
      buildRef: if args["--reference"]: $args["--reference"] else: nil,
      forceRebuild: args["--force-rebuild"],
      workspaceDir: if args["--workspace"]: $args["<workspace>"] else: mkdtemp())

    try:
      let status = core.runStep(cfg, req, logProcOutput)
      if status.state == "failed": raiseEx status.details
      echo "strawboss: build passed."
    except:
      echo "strawboss: build FAILED: " & getCurrentExceptionMsg() & "."
      quit(QuitFailure)
    finally:
      if existsDir(req.workspaceDir): removeDir(req.workspaceDir)

  elif args["serve"]: server.start(cfg)

